/*
 * $Id: IkasanSecurity.java 1671 2009-06-05 17:01:24Z karianna $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasan-0.7.7/common/src/main/java/org/ikasan/common/configuration/IkasanSecurity.java $
 * 
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * Copyright (c) 2003-2008 Mizuho International plc. and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the 
 * Free Software Foundation Europe e.V. Talstrasse 110, 40217 Dusseldorf, Germany 
 * or see the FSF site: http://www.fsfeurope.org/.
 * ====================================================================
 */
package org.ikasan.common.configuration;

import java.io.InputStream;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;
import com.thoughtworks.xstream.io.xml.XppDriver;

/**
 * Java representation of the Ikasan Security XML configuration file.
 * 
 * @author Ikasan Development Team
 */
public class IkasanSecurity
    extends AbstractIkasan
{

    /**
     * Default constructor
     */
    public IkasanSecurity()
    {
        // Do Nothing
    }

    /**
     * Constructor: creates new <code>IkasanSecurity</code> instance
     * 
     * @param noNamespaceSchemaLocation
     */
    public IkasanSecurity(String noNamespaceSchemaLocation)
    {
        this.noNamespaceSchemaLocation = noNamespaceSchemaLocation;
    }

    /**
     * Returns a string representation of this object.
     */
    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();
        sb.append("Ikasan Security [");

        // String values of each entry
        StringBuilder sbEntries = new StringBuilder();
        for(Entry entry : this.getEntries())
            sbEntries.append(entry.toString());

        sb.append(sbEntries);
        sb.append("].");
        return sb.toString();
    }

    /**
     * Equality test
     * @param ikasanSecurity 
     * @return boolean
     */
    public boolean equals(final IkasanSecurity ikasanSecurity)
    {
        if (this.getEntries() == null && ikasanSecurity.getEntries() == null)
        {
            // compare each entry with each entry in the incoming entries
            for(int x=0; x < ikasanSecurity.getEntries().size(); x++)
            {
                // return false if any do not match
                if( !(this.entries.get(x).equals(ikasanSecurity.getEntries().get(x))) )
                    return false;
            }

            // if we're here they must all match
            return true;
        }

        // something was null and therefore no match
        return false;
    }

    /**
     * Converts the object to an XML string
     * 
     * @return resulting XML string
     */
    public String toXML()
    {
        XStream xstream = new XStream(new XppDriver(new XmlFriendlyReplacer(
            "$", "_")));
        setXstreamProps(xstream);
        return xstream.toXML(this);
    }

    /**
     * Converts an incoming XML string to an object
     * 
     * @param xml XML string
     * @return the IkasanSecurity instance
     */
    public static IkasanSecurity fromXML(final String xml)
    {
        XStream xstream = new XStream(new DomDriver());
        setXstreamProps(xstream);
        return (IkasanSecurity) xstream.fromXML(xml);
    }

    /**
     * Converts an incoming XML string to an object
     * 
     * @param xml XML input stream
     * @return the IkasanSecurity instance
     */
    public static IkasanSecurity fromXML(final InputStream xml)
    {
        XStream xstream = new XStream(new DomDriver());
        setXstreamProps(xstream);
        return (IkasanSecurity) xstream.fromXML(xml);
    }

    /**
     * Sets the common properties for the toXML/fromXML XStream object
     * 
     * @param xstream
     */
    private static void setXstreamProps(XStream xstream)
    {
        xstream.registerConverter(new IkasanSecurityConverter());
        xstream.alias(IkasanSecurity.class.getSimpleName(), IkasanSecurity.class);
        xstream.registerConverter(new EntryConverter());
        xstream.alias(Entry.class.getSimpleName(), Entry.class);
    }

}
